package org.openstreetmap.josm.plugins.lanetool.view;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.JPopupMenu;

import org.openstreetmap.josm.plugins.lanetool.data.Lane;
import org.openstreetmap.josm.plugins.lanetool.data.LaneGroup;
import org.openstreetmap.josm.plugins.lanetool.view.style.LanesStyle;
import org.openstreetmap.josm.plugins.lanetool.view.util.ImageUtil;

/**
 * view representation of a lane group
 */
class LaneGroupView {

	private LaneGroup laneGroup;
	
	private LanesStyle style;
	
	private Map<Integer, LaneView> sublaneViews;
	
	private Rectangle laneRect;

	private int xOffset;
	private int yMin;
	private int yMax;
	
	public void setXOffset(int xOffset) {
		this.xOffset = xOffset;
		determineRects();
		updateSublaneXOffsets();
	}

	public LaneGroupView(LaneGroup visualizedLaneGroup, LanesStyle style, int yMin, int yMax) {
		
		this.laneGroup = visualizedLaneGroup;
		
		this.style = style;
		
		this.yMin = yMin;
		this.yMax = yMax;
		
		this.sublaneViews = new HashMap<Integer, LaneView>();
		
		for (Lane sublane : laneGroup.getLanes()) {				
			Integer sublanePosition = laneGroup.getLanePosition(sublane);
			sublaneViews.put(sublanePosition, new LaneView(sublane, style, yMin, yMax));
		}
		
		setXOffset(0);		
		
	}
	
	private void determineRects() {
		
		laneRect = new Rectangle(
				xOffset - getLaneGroupWidth()/2,
				yMin,
				getLaneGroupWidth(),
				yMax);

	}
	
	private int getLaneGroupWidth() {
		
		if (sublaneViews.size() == 0) {
		
			return style.getLaneImage(null).getWidth(null);
				
		} else {
			
			int laneGroupWidth = 0;
			
			for (LaneView sublaneView : sublaneViews.values()) {
				laneGroupWidth += sublaneView.getLaneWidth();
			}
	
			return laneGroupWidth;
			
		}
		
	}
	
	private void updateSublaneXOffsets() {
		
		int sublaneXOffset = xOffset - getLaneGroupWidth()/2;
		
		List<Integer> sublanePositions = new LinkedList<Integer>(); 
		sublanePositions.addAll(sublaneViews.keySet());
		Collections.sort(sublanePositions);
		
		for (int sublanePosition : sublanePositions) {
			LaneView sublaneView = sublaneViews.get(sublanePosition);				
			sublaneXOffset += sublaneView.getLaneWidth()/2;
			sublaneView.setXOffset(sublaneXOffset);				
			sublaneXOffset += (sublaneView.getLaneWidth()+1)/2;
		}
		
	}

	/**
	 * handles a left click on the lanes panel by checking whether this lane group is affected
	 * 
	 * @return  true if the click was handled
	 */
	public boolean handleLeftClick(Component invoker, int x, int y, boolean shift, boolean ctrl) {

		if (laneRect.contains(x, y)) {
		
			boolean handled = false;
			
			for (LaneView sublaneView : sublaneViews.values()) {
				handled |= sublaneView.handleLeftClick(invoker, x, y, shift, ctrl);
				if (handled) { break; }
			}
						
			return handled;
		
		} else {
			
			return false;
			
		}
		
	}
	
	/**
	 * handles a right click on the lanes panel by checking whether this lane group is affected
	 * 
	 * @return  true if the click was handled
	 */
	public boolean handleRightClick(Component invoker, int x, int y, boolean shift, boolean ctrl) {
		
//		TODO: decide about right-click area
//		if (laneRect.contains(x, y)) {
		
			boolean handled = false;

			if (sublaneViews.size() == 0) {
				
				JPopupMenu popup = new LaneGroupPopupMenu(laneGroup);
				popup.show(invoker, x, y);
				
				return true;
				
			} else {
					
				for (LaneView sublaneView : sublaneViews.values()) {
					handled |= sublaneView.handleRightClick(invoker, x, y, shift, ctrl);
					if (handled) { break; }
				}
				
			}
			
			return handled;
		
//		} else {
//			
//			return false;
//			
//		}
		
	}
	
	/**
	 * recursively paints this lane and all sublanes
	 */
	public void paint(Graphics g) {
		
		if (sublaneViews.size() == 0) {
			Image noLaneImage = style.getLaneImage(null);
			ImageUtil.fillWithImage(noLaneImage, g, laneRect);			
		} else {
			for (LaneView sublaneView : sublaneViews.values()) {
				sublaneView.paint(g);
			}
		}
		
	}
	
}
